#!/usr/bin/perl
#
# (C) Copyright 2013, 2019 Hewlett Packard Enterprise Development LP
#
# This program summarizes the basic output of dlook (not the physical
# page view).
# It produces a per-node page allocation total for each
# section of the memory map, as well as an overall summary for
# the entire memory map.
# It could be rendered unnecessary by adding a summary option to dlook.
#
# usage  dlook.pl <pid>
#
# original author: jbaron
 
@out = `dlook @ARGV 2>&1`;
 
$new_object = 1;
foreach(@out) {
  chomp;
  print "$_\n";
  if ($_ =~ /on node/) {
    if ($new_object == 1) {
      $new_object = 0;
    }
    @f = split;
    if ($f[2] =~ /hpage/) {
      $node{$f[5]} = 1;
      $overallnode{$f[5]} = 1;
      $hsum{$f[5]} += $f[1];
      $hoverall{$f[5]} += $f[1];
    } else {
      $node{$f[5]} = 1;
      $overallnode{$f[5]} = 1;
      $sum{$f[5]} += $f[1];
      $overall{$f[5]} += $f[1];
    }
  } else {
    if ($new_object == 0) {
      $new_object = 1;
      printf("Totals:\n");
      foreach $key (sort numerically (keys(%node))) {
        if ($sum{$key} > 0) {
          printf("\t%48d  pages on node %3d\n", $sum{$key}, $key);
        }
        if ($hsum{$key} > 0) {
          printf("\t%48d hpages on node %3d\n", $hsum{$key}, $key);
        }
        delete $sum{$key};
        delete $hsum{$key};
        delete $node{$key};
      }
      printf("\n\n");
    }
  }
}

printf("Overall:\n");
foreach $key (sort numerically (keys(%overallnode))) {
  if ($overall{$key} > 0) {
    printf("TOTAL\t%48d  pages on node %3d\n", $overall{$key}, $key);
  }
  if ($hoverall{$key} > 0) {
    printf("TOTAL\t%48d hpages on node %3d\n", $hoverall{$key}, $key);
  }
  delete $overall{$key};
  delete $hoverall{$key};
  delete $overallnode{$key};
}
 
sub numerically { $a <=> $b; }

